@@ -19,6 +19,8 @@ local core          = require("apisix.core")
 local config_util   = require("apisix.core.config_util")
 local enable_debug  = require("apisix.debug").enable_debug
 local wasm          = require("apisix.wasm")
+local ngx           = ngx
+local crc32         = ngx.crc32_short
 local ngx_exit      = ngx.exit
 local pkg_loaded    = package.loaded
 local sort_tab      = table.sort
@@ -27,7 +29,6 @@ local ipairs        = ipairs
 local pairs         = pairs
 local type          = type
 local local_plugins = core.table.new(32, 0)
-local ngx           = ngx
 local tostring      = tostring
 local error         = error
 local is_http       = ngx.config.subsystem == "http"
@@ -466,6 +467,10 @@ local function merge_service_route(service_conf, route_conf)
         new_conf.value.host = route_conf.value.host
     end
 
+    if route_conf.value.labels then
+        new_conf.value.labels = route_conf.value.labels
+    end
+
     -- core.log.info("merged conf : ", core.json.delay_encode(new_conf))
     return new_conf
 end
@@ -497,6 +502,9 @@ local function merge_consumer_route(route_conf, consumer_conf)
             new_route_conf.value.plugins = {}
         end
 
+        if new_route_conf.value.plugins[name] == nil then
+            conf._from_consumer = true
+        end
         new_route_conf.value.plugins[name] = conf
     end
 
@@ -546,13 +554,13 @@ end
 
 
 function _M.init_worker()
+    _M.load()
+
     -- some plugins need to be initialized in init* phases
-    if ngx.config.subsystem == "http" then
+    if ngx.config.subsystem == "http" and local_plugins_hash["prometheus"] then
         require("apisix.plugins.prometheus.exporter").init()
     end
 
-    _M.load()
-
     if local_conf and not local_conf.apisix.enable_admin then
         init_plugins_syncer()
     end
@@ -599,6 +607,19 @@ function _M.get_all(attrs)
 end
 
 
+-- conf_version returns a version which only depends on the value of conf,
+-- instead of where this plugin conf belongs to
+function _M.conf_version(conf)
+    if not conf._version then
+        local data = core.json.stably_encode(conf)
+        conf._version = tostring(crc32(data))
+        core.log.info("init plugin-level conf version: ", conf._version, ", from ", data)
+    end
+
+    return conf._version
+end
+
+
 local function check_schema(plugins_conf, schema_type, skip_disabled_plugin)
     for name, plugin_conf in pairs(plugins_conf) do
         core.log.info("check plugin schema, name: ", name, ", configurations: ",
@@ -712,8 +733,13 @@ function _M.run_plugin(phase, plugins, api_ctx)
     if phase ~= "log"
         and phase ~= "header_filter"
         and phase ~= "body_filter"
+        and phase ~= "delayed_body_filter"
     then
         for i = 1, #plugins, 2 do
+            if phase == "rewrite_in_consumer" and plugins[i + 1]._from_consumer
+                    and plugins[i].type ~= "auth"then
+                phase = "rewrite"
+            end
             local phase_func = plugins[i][phase]
             if phase_func then
                 plugin_run = true
